#! /usr/bin/perl -w
#--------------------
# server.pl
# Stream2Dream Linux server. This is the replacement for the .Net version of windows.
# It needs perl, samba and vlc.
# Current version: 2.0 (08-03-2008)
#
# Change log:
# ===========
#
# 15-03-2008:
# Updated the server to version 2.0. Based on the LT Team 4 Revolution image.
# Added youtube video
# Removed develop debug
#
# 23-12-2007:
# Fixed the VLC pid detection and kill system. VLC should now be stoppped when you stop the movie on the Dreambox. Thanks to XX9yCDAS for pointing it out
#
# 16-12-2007:
# Added a trim function and a better vlc pid detecton and kill system. Now all VLC instances should be killed when the movie is stoped
#
# 24-11-2007:
# Added the option to use an alternative codec. mp2v videocodec has been renamed to mpgv on newer linux systems
#
# 07-11-2007:
# Fixed a vlc transcode error. Thanks to Micke for pointing it out!
# Added a vlc pid detection part for better control over what process to kill on the end
# Added the option to turn of the debug
# Added an option to turn of the interface of VLC. This overrules the dreambox setting
#
# 28-10-2007:
# Rewritten the code to the new version 1.7. Third challange image
# First public version
#
# 22-07-2007:
# Initial version based on the second LT image (player version 1.5)
# Made by TheYOSH (http://theyosh.nl/projects/stream2dream-linux/stream2dream-linux - dreambox@theyosh.nl)
#--------------------

use strict;
use IO::Socket;
use Switch;
use Date::Format;
use LWP::Simple;

# Settings
#Normal port number (should not be changed)
my $port = 9916;
# The IP of the streaming server (Linux box)
my $server = "192.168.5.1";
# The name of the samba share (Not the location)
my $share = "film";
# Change the outputshare to a different place
my $outputshare = $share;
# The location on the disk
my $location = "/mnt/film";
# Output location
my $outputlocation = $location;
# Alternative video codec (If VLC has problems coding video.) Set to 1 to activate
my $alternativecode = 1;
# Location of VLC
my $vlc = "/usr/bin/vlc";
# Default vlc gui, set to dummy to overrule the Dreambox setting
my $vlcgui = "dummy";
# Debug modus, set to 0 to turn debug of
my $debug = 0;

# System variables, should not be changed!
my $transform = "";
#my $httpHeader = 0;
my @pids = ();

# Start server, open network connection for listening
my $sock = new IO::Socket::INET(
	LocalHost => $server,
	LocalPort => $port,
	Proto => 'tcp',
	Listen => SOMAXCONN,
	Reuse => 1);

$sock or die debugMsg("TCP-Server","no socket: $!");
my($new_sock, $c_addr, $buf);

debugMsg("TCP-Server","Starting listener...");
debugMsg("Info","The 'LT Stream-Player' can connect to us now :-)");

# Loop unlimited times for server requests
while (($new_sock, $c_addr) = $sock->accept())
{
	my ($client_port, $c_ip) =sockaddr_in($c_addr);
	my $client_ipnum = inet_ntoa($c_ip);
	my $client_host = gethostbyaddr($c_ip, AF_INET);
	debugMsg("TCP-Server","Client: $client_ipnum logged in");

	while (defined ($buf = <$new_sock>))
	{
		$buf = trim($buf);
		my $cmdnr = substr($buf,0,1);
		my $output = "";
		my $vlccmd = "";

		switch ($cmdnr) {
			case 0 {
				# Set the transcode settings
				my $streamfile = "\"" . $outputlocation . "/MPStream.ts\"";
				my $start = index($buf,"\$-I")+3;
				if ($vlcgui ne "dummy") { $vlcgui = substr($buf,$start,index($buf," \$\$ ")-$start); }
				$transform = substr($buf,index($buf,":sout"));
				$transform =~ s/dst=\$/dst=$streamfile/;
				if ($alternativecode > 0) { $transform =~ s/mp2v/mpgv/; } # Dreambox wants mp2v codec, but codec has been changed to mpgv in newer linux versions)
				$output = "ok\$2.0\$mount -t cifs //\$/$share /tmp/mp -o ro,nolock,rsize=\$r,wsize=\$w,user=\$u,password=\$p; mount -t cifs //\$/$outputshare /tmp/mpstream -o ro,nolock,rsize=\$r,wsize=\$w,user=\$u,password=\$p###";
			}
			case 1 {
				# Play the selected movie
				$vlccmd = $vlc . " -I $vlcgui ";
				my $file = substr($buf,1);
				if ( index($file,"youtube") != -1 ) {
					# YouTube
					$file = getYouTubeVideoUrl($file);
					$vlccmd .= "\"" . $file . "\" ";
				} else {
					# Default operation
					$vlccmd .= "\"file://" . $location . "/" . $file . "\" ";
				}
				$vlccmd .= "\"" . $transform . "\" ";
				$vlccmd = $vlccmd . "> /dev/null 2>/dev/null ";
				#debugMsg("vlc","CMD: $vlccmd");
				$vlccmd = $vlccmd . "&";
				$output = "ok###";
				system($vlccmd);
				my $getVLCpidCMD = "ps fax -o pid -o cmd | grep vlc | grep \"" . $file . "\" | grep -v grep";
				debugMsg("vlc","Movie: $file");
				my $data = readpipe $getVLCpidCMD;
				my $pid;
				foreach $pid (split(/\n/,$data)) {
					my @piddata = split(/ /,trim($pid));
					push(@pids,$piddata[0]);
				}
			}
			case 2 {
				# Exit current movie...
				my $pid;
				foreach $pid (@pids) {
					system("kill " . $pid . " &");
				}
				@pids = ();
				$output = "ok###";
			}
			case 9 {
				debugMsg("TCP-Server","Client: $client_ipnum logged out");
				$output = "ok###";
				system("rm -f $outputlocation/MPStream.ts");
			}
		}
		print $new_sock $output;
	}
}

sub getYouTubeVideoUrl {
	my $lWatchLocation = $_[0];
	my $YouTubeContent = get $lWatchLocation;
	my $lStart = index($YouTubeContent,", \"t\": \"");
	$lStart += 8;
	my $lStop =  index($YouTubeContent,"\", \"hl\":");
	my $cookieID = substr($YouTubeContent,$lStart,$lStop-$lStart);

	$lStart = index($lWatchLocation,"watch?v=");
	$lStart += 8;
	my $videourl = "http://youtube.com/get_video?video_id=" . substr($lWatchLocation,$lStart) . "&t=" . $cookieID;
	return $videourl;
}

sub debugMsg {
	my ($pSystem,$pMessage) = ($_[0], $_[1]);
	print time2str("%d-%m-%Y %H:%M:%S",time) . " [$pSystem] $pMessage\n";
}

sub trim
{
	my $string = $_[0];
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}